==> cpl-plugin-muse_0.18.5%2Bdfsg.orig.tar.xz: muse-0.18.5/tools/muse_pixtable_erase_slice.c <==
/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim:set sw=2 sts=2 et cin: */
/*
 * This file is part of the MUSE Instrument Pipeline
 * Copyright (C) 2014 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <muse.h>
#include <string.h>

/*----------------------------------------------------------------------------*/
/**
 * @defgroup tools_musepixtableeraseslice   Tool muse_pixtable_erase_slice
 *
 * @brief   <b>muse_pixtable_erase_slice</b>: Erase a slice of one IFU from
 *                                                a MUSE pixel table.
 *
 * <b>Command line arguments:</b>
 *   - <tt><b>PIXTABLE_IN</b></tt>\n
 *     the filename of the pixel table to erase a slice in
 *   - <tt><b>IFU</b></tt>\n
 *     IFU number of the slice to erase
 *   - <tt><b>SLICE</b></tt>\n
 *     SLICE number (CCD numbering) of the slice (belonging to IFU) to erase
 *   - <tt><b>PIXTABLE_OUT</b></tt>\n
 *     the filename of the output pixel table to write; can be the same as
 *     PIXTABLE_IN which causes the input table to be overwritten
 *
 * <b>Return values:</b>
 *   - <tt> 0</tt>\n   Success
 *   - <tt> 1</tt>\n   not four arguments given
 *   - <tt> 2</tt>\n   illegal IFU number given
 *   - <tt> 3</tt>\n   illegal slice number given
 *   - <tt>10</tt>\n   pixel table could not be loaded from file
 *   - <tt>50</tt>\n   unknown error
 */
/*----------------------------------------------------------------------------*/

/**@{*/

#define PRINT_USAGE(rc)                                                        \
  fprintf(stderr, "Usage: %s PIXTABLE_IN IFU SLICE PIXTABLE_OUT\n", argv[0]);  \
  cpl_end(); return (rc);

/*---------------------------------------------------------------------------*/
/**
  @brief   Fix the exposure ranges in the header of a pixel table.
  @param   aPixtable the pixel table
  @retval  CPL_ERROR_NONE on success, another cpl_error_code on failure.

  This function assumes that all selected rows are going to be erased, and
  resets the header keywords MUSE_HDR_PT_EXP_FST and MUSE_HDR_PT_EXP_LST
  accordingly.

  XXX temporarily added directly here, in the tool, to not mess up the API
 */
/*---------------------------------------------------------------------------*/
static cpl_error_code
muse_pixtable_fix_exp_headers(muse_pixtable *aPixtable)
{
  cpl_ensure_code(aPixtable && aPixtable->header && aPixtable->table,
                  CPL_ERROR_NULL_INPUT);
  if (cpl_table_count_selected(aPixtable->table) < 1) {
    return CPL_ERROR_NONE; /* nothing to do */
  }
  cpl_array *sel = cpl_table_where_selected(aPixtable->table);
  cpl_size narray = cpl_array_get_size(sel),
           nselprev = 0, /* selected ones in previous exposures */
           ifst = 0, ilst = 0;
  const cpl_size *asel = cpl_array_get_data_cplsize_const(sel);
  unsigned int nexp = 0;
  do {
    /* get exposure range for (next) exposure */
    char *kwfst = cpl_sprintf(MUSE_HDR_PT_EXP_FST, ++nexp),
         *kwlst = cpl_sprintf(MUSE_HDR_PT_EXP_LST, nexp);
    if (!cpl_propertylist_has(aPixtable->header, kwfst) ||
        !cpl_propertylist_has(aPixtable->header, kwlst)) {
      cpl_free(kwfst);
      cpl_free(kwlst);
      break; /* nothing (more) to do */
    }
    ifst = cpl_propertylist_get_long_long(aPixtable->header, kwfst);
    ilst = cpl_propertylist_get_long_long(aPixtable->header, kwlst);
    /* count selected */
    cpl_size i, nsel = 0;
    for (i = 0; i < narray; i++) {
      if (asel[i] >= ifst && asel[i] <= ilst) {
        nsel++;
      } /* if in range of this exposure */
    } /* for i (all array entries) */
    cpl_size ifst2 = ifst - nselprev,
             ilst2 = ilst - nsel - nselprev;
    cpl_msg_debug(__func__, "exp %d old %"CPL_SIZE_FORMAT"..%"CPL_SIZE_FORMAT
                  ", %"CPL_SIZE_FORMAT" selected (previous: %"CPL_SIZE_FORMAT
                  "), new %"CPL_SIZE_FORMAT"..%"CPL_SIZE_FORMAT, nexp,
                  ifst, ilst, nsel, nselprev, ifst2, ilst2);
    /* finally update the header entries */
    muse_cplpropertylist_update_long_long(aPixtable->header, kwfst, ifst2);
    muse_cplpropertylist_update_long_long(aPixtable->header, kwlst, ilst2);
    cpl_free(kwfst);
    cpl_free(kwlst);
    nselprev += nsel; /* add selected ones for previous exposures */
  } while (ilst >= ifst);
  cpl_array_delete(sel);
  return CPL_ERROR_NONE;
} /* muse_pixtable_fix_exp_headers() */

/*---------------------------------------------------------------------------*/
/**
  @brief   Erase pixel table rows related to one slice of one IFU
  @param   aPixtable   the pixel table
  @param   aIFU        the IFU number
  @param   aSlice      the slice number
  @return  CPL_ERROR_NONE on success, another CPL error code on failure.

  This function selects all rows in the given pixel table which originated in
  aIFU and aSlice. Then all selected slices are erased.

  XXX temporarily added directly here, in the tool, to not mess up the API
      to become a real function in muse_pixtable on the trunk

  @error{return CPL_ERROR_NULL_INPUT, aPixtable is NULL}
  @error{return CPL_ERROR_DATA_NOT_FOUND, no rows found in aPixtable}
  @error{return CPL_ERROR_ILLEGAL_INPUT,
         aPixtable contains data from more than one exposure}
 */
/*---------------------------------------------------------------------------*/
static cpl_error_code
muse_pixtable_erase_ifu_slice(muse_pixtable *aPixtable, unsigned char aIFU,
                              unsigned short aSlice)
{
  cpl_ensure_code(aPixtable, CPL_ERROR_NULL_INPUT);
  cpl_size nrow = muse_pixtable_get_nrow(aPixtable);
  cpl_ensure_code(nrow > 0, CPL_ERROR_DATA_NOT_FOUND);

  cpl_table_unselect_all(aPixtable->table);
  uint32_t *origin = (uint32_t *)cpl_table_get_data_int(aPixtable->table,
                                                        MUSE_PIXTABLE_ORIGIN);
  cpl_size irow;
  for (irow = 0; irow < nrow; irow++) {
    unsigned char ifu = muse_pixtable_origin_get_ifu(origin[irow]);
    unsigned short slice = muse_pixtable_origin_get_slice(origin[irow]);
    if (ifu == aIFU && slice == aSlice) {
      cpl_table_select_row(aPixtable->table, irow);
    } /* if same IFU and slice */
  } /* for irow (all pixtable rows) */
  cpl_size nsel = cpl_table_count_selected(aPixtable->table);
  cpl_error_code rc = cpl_table_erase_selected(aPixtable->table);
  cpl_msg_debug(__func__, "Erased %"CPL_SIZE_FORMAT" rows from pixel table",
                nsel);

  muse_pixtable_fix_exp_headers(aPixtable);
  muse_pixtable_compute_limits(aPixtable);
  return rc;
} /* muse_pixtable_erase_ifu_slice() */

int main(int argc, char **argv)
{
  cpl_init(CPL_INIT_DEFAULT);
  cpl_errorstate prestate = cpl_errorstate_get();

  if (argc != 5) {
    /* four arguments are needed */
    PRINT_USAGE(1);
  }

  char *tiname = NULL, /* input table */
       *toname = NULL; /* output table */
  unsigned char ifu = 0;
  unsigned short slice = 0;
  int i;

  /* argument processing */
  for (i = 1; i < argc; i++) {
    if (tiname && toname) {
      break; /* we have the required name, skip the rest */
    }
    if (!tiname) {
      tiname = argv[i]; /* set the name for the input table */
    } else if (!ifu) {
      ifu = atoi(argv[i]);
    } else if (!slice) {
      slice = atoi(argv[i]);
    } else {
      toname = argv[i]; /* set the name for the output table */
    }
  } /* for i (all arguments) */
  if (ifu == 0 || ifu > kMuseNumIFUs) {
    printf("Illegal IFU number %hhu given\n", ifu);
    PRINT_USAGE(2);
  }
  if (slice == 0 || slice > kMuseSlicesPerCCD) {
    printf("Illegal slice number %hhu given\n", slice);
    PRINT_USAGE(3);
  }

  muse_pixtable *table = muse_pixtable_load(tiname);
  if (!table) {
    PRINT_USAGE(10);
  }
  cpl_size nrow = muse_pixtable_get_nrow(table);
  printf("Loaded pixel table \"%s\" with %"CPL_SIZE_FORMAT" rows\n",
         tiname, nrow);

  cpl_error_code rc = muse_pixtable_erase_ifu_slice(table, ifu, slice);
  cpl_size nrow2 = muse_pixtable_get_nrow(table);
  if (rc == CPL_ERROR_NONE) {
    printf("Erased slice %hu of IFU %hhu (%"CPL_SIZE_FORMAT" pixels removed)\n",
           slice, ifu, nrow - nrow2);
  } else {
    printf("Error while erasing slice %hu of IFU %hhu: %s\n", slice, ifu,
           cpl_error_get_message());
  }

  muse_pixtable_save(table, toname);
  printf("MUSE pixel table \"%s\" (%"CPL_SIZE_FORMAT" rows) saved\n", toname,
         nrow2);
  muse_pixtable_delete(table);
  rc = cpl_errorstate_is_equal(prestate) ? 0 : 50;
  if (rc) {
    cpl_errorstate_dump(prestate, CPL_FALSE, muse_cplerrorstate_dump_some);
  }
  cpl_end();
  return rc;
}

/**@}*/

==> cpl-plugin-muse_1.0.5%2Bdfsg.orig.tar.xz: muse-1.0.5/tools/muse_pixtable_erase_slice.c <==
/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim:set sw=2 sts=2 et cin: */
/*
 * This file is part of the MUSE Instrument Pipeline
 * Copyright (C) 2014 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <muse.h>
#include <string.h>

/*----------------------------------------------------------------------------*/
/**
 * @defgroup tools_musepixtableeraseslice   Tool muse_pixtable_erase_slice
 *
 * <b>muse_pixtable_erase_slice</b>: Erase a slice of one IFU from a MUSE pixel
 *                                   table.
 *
 * <b>Command line arguments:</b>
 *   - <tt><b>PIXTABLE_IN</b></tt>\n
 *     the filename of the pixel table to erase a slice in
 *   - <tt><b>IFU</b></tt>\n
 *     IFU number of the slice to erase
 *   - <tt><b>SLICE</b></tt>\n
 *     SLICE number (CCD numbering) of the slice (belonging to IFU) to erase
 *   - <tt><b>PIXTABLE_OUT</b></tt>\n
 *     the filename of the output pixel table to write; can be the same as
 *     PIXTABLE_IN which causes the input table to be overwritten
 *
 * <b>Return values:</b>
 *   - <tt> 0</tt>\n   Success
 *   - <tt> 1</tt>\n   not four arguments given
 *   - <tt> 2</tt>\n   illegal IFU number given
 *   - <tt> 3</tt>\n   illegal slice number given
 *   - <tt>10</tt>\n   pixel table could not be loaded from file
 *   - <tt>50</tt>\n   unknown error
 */
/*----------------------------------------------------------------------------*/

/**@{*/

#define PRINT_USAGE(rc)                                                        \
  fprintf(stderr, "Usage: %s PIXTABLE_IN IFU SLICE PIXTABLE_OUT\n", argv[0]);  \
  cpl_end(); return (rc);

int main(int argc, char **argv)
{
  cpl_init(CPL_INIT_DEFAULT);
  cpl_errorstate prestate = cpl_errorstate_get();

  if (argc != 5) {
    /* four arguments are needed */
    PRINT_USAGE(1);
  }

  char *tiname = NULL, /* input table */
       *toname = NULL; /* output table */
  unsigned char ifu = 0;
  unsigned short slice = 0;
  int i;

  /* argument processing */
  for (i = 1; i < argc; i++) {
    if (tiname && toname) {
      break; /* we have the required name, skip the rest */
    }
    if (!tiname) {
      tiname = argv[i]; /* set the name for the input table */
    } else if (!ifu) {
      ifu = atoi(argv[i]);
    } else if (!slice) {
      slice = atoi(argv[i]);
    } else {
      toname = argv[i]; /* set the name for the output table */
    }
  } /* for i (all arguments) */
  if (ifu == 0 || ifu > kMuseNumIFUs) {
    printf("Illegal IFU number %hhu given\n", ifu);
    PRINT_USAGE(2);
  }
  if (slice == 0 || slice > kMuseSlicesPerCCD) {
    printf("Illegal slice number %hhu given\n", slice);
    PRINT_USAGE(3);
  }

  muse_pixtable *table = muse_pixtable_load(tiname);
  if (!table) {
    PRINT_USAGE(10);
  }
  cpl_size nrow = muse_pixtable_get_nrow(table);
  printf("Loaded pixel table \"%s\" with %"CPL_SIZE_FORMAT" rows\n",
         tiname, nrow);

  cpl_error_code rc = muse_pixtable_erase_ifu_slice(table, ifu, slice);
  cpl_size nrow2 = muse_pixtable_get_nrow(table);
  if (rc == CPL_ERROR_NONE) {
    printf("Erased slice %hu of IFU %hhu (%"CPL_SIZE_FORMAT" pixels removed)\n",
           slice, ifu, nrow - nrow2);
  } else {
    printf("Error while erasing slice %hu of IFU %hhu: %s\n", slice, ifu,
           cpl_error_get_message());
  }

  muse_pixtable_save(table, toname);
  printf("MUSE pixel table \"%s\" (%"CPL_SIZE_FORMAT" rows) saved\n", toname,
         nrow2);
  muse_pixtable_delete(table);
  rc = cpl_errorstate_is_equal(prestate) ? 0 : 50;
  if (rc) {
    cpl_errorstate_dump(prestate, CPL_FALSE, muse_cplerrorstate_dump_some);
  }
  cpl_end();
  return rc;
}

/**@}*/
